#!/usr/local/bin/perl
#----------------------------------------------------
# calibrate.pl
#----------------------------------------------------
# --Acquires user input about Calibration
# --Invokes calibration procedure, written in "C"
# --Outputs calibration results
# --Plots a graph with probability histogram (a bar chart)
# --Generates form to perform option pricing (i.e. generate option.in)
#----------------------------------------------------


require "/home/B6H/development/deneb_cgi-bin/miloje/cgi_handlers.pl";

$DATA_DIR = "/home/B6H/development/document_root/users/miloje/Demo/Data";
$TMP_DIR = "/home/B6H/development/document_root/users/miloje/Demo/tmp";
$TMP_URL = "http://deneb.npac.syr.edu:3768/users/miloje/Demo/tmp";
$CALIB_ROOT="/home/B8E/miloje/Demo/Home/Calib";
$SCRIPT_ROOT="/home/B6H/development/deneb_cgi-bin/miloje";
$SCRIPT_ROOT_URL = "http://deneb.npac.syr.edu:3768/cgi-bin/miloje";

&html_header("Calibration Setup");
print "<pre>";
&get_request;
$pid = $$;
$symbol = $rqpairs{'symbol'};
print "$symbol\n";
$hfile = "$DATA_DIR/history.$symbol";
print "$hfile\n";
if($rqpairs{'symbol'} ne "") {
  $_ = `wc -l $hfile`;
  ($ndata, $junk) = /(\d+) (\w+)/;
} else {
  print "Could not find the symbol, try again"
};

$PriceType = 1;
print "$PriceType\n";
$LastClosingPrice = $rqpairs{'last_closing'};
$delT = $rqpairs{'time_unit'};
$NumData = $rqpairs{'ndata'};
$NumHistBins= $rqpairs{'nhbins'};
$NumSD = $rqpairs{'nsd'};
$DistType = $rqpairs{'dtype'};
print "$DistType\n";

$gnu_file = "$symbol.$pid.gnu";
$plot_hdata = "hist.$symbol.$pid.dat";
$plot_fdata = "fit.$symbol.$pid.dat";
$ppm_chartfile = "chart.$symbol.$pid.ppm";
$gif_chartfile = "chart.$symbol.$pid.gif";
$gif_chartURL  = "$TMP_URL/chart.$symbol.$pid.gif";
print "$gif_chartURL\n";

$Histogram = "hgram.$pid.dat";
$Param = "param.$pid.dat";
#eval `rm -f $TMP_DIR/$Histogram`;
#eval `rm -f $TMP_DIR/$Param`;

print "calib $hfile $Histogram $delT $NumData $PriceType $NumHistBins $NumSD $DistType $Param\n";
eval `cd $TMP_DIR; ./bin/calib $hfile $Histogram $delT $NumData $PriceType $NumHistBins $NumSD $DistType $Param`;
print "finished calib\n";

open(OUT_HIST,"$TMP_DIR/$Histogram") || die "Cannot open calibration data file";
print "opened $Histogram\n";
open(OUT_PARAM,"$TMP_DIR/$Param") || die "Cannot open volatility parameter file";
print "opened $Param\n";
($par{'drift'},$par{'volatility'}) = <OUT_PARAM>;
close(OUT_PARAM);
$mu = $par{'drift'};
$sigma = $par{'volatility'};
$coupling = 1./($sigma * $sigma);
print "coupling=$coupling\n";

open(HISTOGRAM_DATA, ">$TMP_DIR/$plot_hdata") || die "couldn't open $plot_hdata";
open(FIT_DATA, ">$TMP_DIR/$plot_fdata") || die "couldn't open $plot_fdata";
print "opened $TMP_DIR/$plot_hdata and $TMP_DIR/$plot_fdata\n";
$index=0;
while(<OUT_HIST>) {
 $index++;
 ($x, $hist, $fit)=split;
 print(HISTOGRAM_DATA "$x $hist\n");
 print(FIT_DATA "$x $fit\n");
}
close(HISTOGRAM_DATA);
close(FIT_DATA);

close(OUT_HIST);

#NOTE: gnuplot expects "pbm", even though it actually writes out a PPM file
open(OUTPUT, ">$TMP_DIR/$gnu_file") || die "couldn't open $gnu_file";
print(OUTPUT "set term pbm small\n");
# the default size 1, 1 produces a 640x480 size chart...
print(OUTPUT "set size 0.72, 0.54\n");
print(OUTPUT "cd \"$TMP_DIR\" \n");
print(OUTPUT "set output \"$ppm_chartfile\" \n");
print(OUTPUT "set title \"Histogram of log-returns for $symbol\" \n");
print(OUTPUT "set grid\n");
print(OUTPUT "plot \"$plot_hdata\", \"$plot_fdata\" \n");
close(OUTPUT);

eval `rm -f $TMP_DIR/$ppm_chartfile 2>/dev/null`;
eval `rm -f $TMP_DIR/$gif_chartfile 2>/dev/null`;
eval `gnuplot $TMP_DIR/$gnu_file 2>/dev/null`;
eval `/home/B8E/miloje/netpbm/bin/ppmtogif $TMP_DIR/$ppm_chartfile > $TMP_DIR/$gif_chartfile 2>/dev/null`;

#eval `cd $TMP_DIR ; rm -f *.ppm *.dat *.gnu tmpPar.* 2>/dev/null`;

print "<hr>";
print "<h1> Histogram and Fitted Distribution </h1>";
print "Volatility: $sigma";
print "<br>";
print "<img src=\"$gif_chartURL\">";
print "<hr>";
print "<h2>Option Valuation</h2>";
print "<hr>";
print "<FORM METHOD=POST ACTION=$SCRIPT_ROOT_URL/option.cgi>";
print "<INPUT TYPE=hidden NAME=symbol VALUE=$rqpairs{'symbol'}>";
print "<INPUT TYPE=hidden NAME=coupling VALUE=$coupling>";
print "<INPUT TYPE=hidden NAME=distribution VALUE=$DistType>";
print "Machine size:";
print <<STOPNP;
               <SELECT NAME="nprocs">
               <OPTION VALUE="8"> 8 processors
               <OPTION VALUE="16"> 16 processors
               <OPTION VALUE="32"> 32 processors
               <OPTION VALUE="64"> 64 processors
               <OPTION VALUE="128"> 128 processors
               <OPTION VALUE="256"> 256 processors
            </SELECT> 
STOPNP
print "Monte Carlo samples per node:"; 
print <<STOP;
               <SELECT NAME="blocks">
               <OPTION VALUE="1000"> 1,000
               <OPTION VALUE="5000"> 5,000
               <OPTION VALUE="10000"> 10,000
               <OPTION VALUE="50000"> 50,000
               <OPTION VALUE="100000"> 100,000
               <OPTION VALUE="500000"> 500,000
               <OPTION VALUE="1000000"> 1,000,000
            </SELECT> 
STOP
print "<hr>";
print "Periods to maturity:";
print "<INPUT TYPE=text NAME=nt SIZE=5 VALUE=12>";
print "Interest rate:";
print "<INPUT TYPE=text NAME=rate SIZE=10 VALUE=0.06>";
print "<hr>";
print "Initial stock prices:";
print "<INPUT TYPE=text NAME=prices SIZE=50 VALUE=$LastClosingPrice>";
print "<hr>";
print "Strike prices:";
print "<INPUT TYPE=text NAME=strikes SIZE=50 VALUE=$LastClosingPrice>";
print "<hr>";

print "<b><INPUT TYPE=SUBMIT VALUE=Price></b> <INPUT TYPE=RESET VALUE=Cancel>";
print "</body>";
print "</html>";
exit;

